import click

from .requirements import Requirements

@click.group()
def cli():
    cli.rqr = Requirements()

@cli.command()
def list():
    click.echo(cli.rqr)

@cli.command()
@click.argument('pkg')
@click.option('--save', 'target', flag_value='base')
@click.option('--save-development', 'target', flag_value='development')
@click.option('--save-production', 'target', flag_value='production')
def install(pkg, target = None):
    click.echo(cli.rqr.install(pkg, target))



import requests
from packaging.version import Version

from .serializer import serialize

def get_last_version(pkg_name):
    response = requests.get('https://pypi.python.org/pypi/{0}/json'.format(pkg_name))
    pkginfo = response.json()

    releases = pkginfo['releases']
    for release in sorted(releases.keys(), reverse=True):
        version = Version(release)
        if not version.is_prerelease:
            return version

def check_file_for_updates(filename):
    requirements = {'base': {}}
    f = open(filename, 'r')
    line = True
    while line:
        line = f.readline().strip()
        parts = line.split('==')
        if len(parts) is 2:
            pkg_name, version_name = parts
            current_version = Version(version_name)
            latest_version = get_last_version(pkg_name)

            if latest_version > current_version:
                print(pkg_name, current_version, '->', latest_version)
                requirements['base'][pkg_name] = str(latest_version)
            else:
                print(pkg_name, 'ok')
                requirements['base'][pkg_name] = str(current_version)
        elif len(line.strip()) is not 0:
            print('Invalid line', line)
        # else: empty line
    serialize(requirements)



import pip
import yaml

from .updater import get_last_version

FILENAME = 'rqr.yaml'


class Requirements:
    def __init__(self):
        self.pkgs = {}
        self.reload()

    def reload(self):
        try:
            with open(FILENAME, 'r') as stream:
                self.pkgs = yaml.load(stream)
                stream.close()
        except FileNotFoundError:
            self.pkgs = {}

    def add(self, pkg, target, version):
        if target not in self.pkgs:
            self.pkgs[target] = {}
        self.pkgs[target][pkg] = version
        self.save()

    def save(self):
        with open(FILENAME, 'w') as stream:
            yaml.dump(self.pkgs, stream, default_flow_style=False)
            stream.close()

    def install(self, pkg, target):
        version = str(get_last_version(pkg))
        if target:
            self.add(pkg, target, version)
        pip.main(['install', pkg])
        return { pkg: version }

    def __str__(self):
        res = []
        for target in self.pkgs:
            res.append(target + ':')
            for requirement in self.pkgs[target]:
                version = self.pkgs[target][requirement]
                res.append('  - {0}@{1}'.format(requirement, version))
        return '\n'.join(res)
